﻿using System;
using System.Collections.Generic;
using UnityEngine;
using SimpleJSON;
using System.Collections.Generic;
using UnityEngine.Events;
using UnityEngine.UI;

namespace WgSoup {
	public class FreeControllerManager : MVRScript {
		protected Dictionary<string, bool> FCv3_list;
		protected Dictionary<string, string> FCv3_position_list;
		protected Dictionary<string, string> FCv3_rotation_list;

		protected string _lastBrowseDir;
		protected UIDynamicButton btn;


		public override void Init() {
			try {
				if(containingAtom.type!="Person")
                {
                    SuperController.LogError("Please attach to a Person Atom");
                }
				else
				{
					FCv3_list = new Dictionary<string,bool>();
					FCv3_position_list = new Dictionary<string,string>();
					FCv3_rotation_list = new Dictionary<string,string>();
					getControllersList();
					createUIControllerList();

					//create button to apply controller preset
					var Apply_current_preset = CreateButton("Apply current preset", false);
					Apply_current_preset.button.onClick.AddListener(applyCurrentControllerPreset);
					JSONStorableAction Apply_current_preset_Action = new JSONStorableAction("Apply current preset", () => {applyCurrentControllerPreset();});
					RegisterAction(Apply_current_preset_Action);

					//create button to match current controller config
					btn = CreateButton("Match current Config");
					btn.button.onClick.AddListener(() =>
					{
						updateUIControllerList();
					});

					//create Preset save Button
					btn = CreateButton("Save Preset");
					btn.button.onClick.AddListener(() =>
					{
						SuperController.singleton.NormalizeMediaPath(_lastBrowseDir);
						SuperController.singleton.GetMediaPathDialog(SavePreset, "FreeControllerManager");

					uFileBrowser.FileBrowser browser = SuperController.singleton.mediaFileBrowserUI;
						browser.SetTextEntry(true);
						browser.fileEntryField.text = System.String.Format("{0}.{1}", ((int)(System.DateTime.UtcNow - new System.DateTime(1970, 1, 1)).TotalSeconds).ToString(), "FreeControllerManager");
						browser.ActivateFileNameField();
					});

					//create Preset load Button
					btn = CreateButton("Load Preset");
					btn.button.onClick.AddListener(() =>
					{
						SuperController.singleton.NormalizeMediaPath(_lastBrowseDir);
						SuperController.singleton.GetMediaPathDialog(LoadPreset, "FreeControllerManager");
					});

					//buttons to use toggles presets

					/*
					btn = CreateButton("Enable all controllers");
					btn.button.onClick.AddListener(() =>
					{
						enableAllControllers();
					});

					btn = CreateButton("Disable all controllers");
					btn.button.onClick.AddListener(() =>
					{
						disableAllControllers();
					});

					CreateTextInput(new JSONStorableString("Presets", "Presets..."), false);
					*/
					btn = CreateButton("Set ON all Controllers");
					btn.button.onClick.AddListener(() =>
					{
						setAllControllersOn();
					});
					JSONStorableAction setAllControllersOn_Action = new JSONStorableAction("Set ON all Controllers", () => {setAllControllersOn();applyCurrentControllerPreset();});
					RegisterAction(setAllControllersOn_Action);

					btn = CreateButton("Set OFF all Controllers");
					btn.button.onClick.AddListener(() =>
					{
						setAllControllersOff();
					});
					JSONStorableAction setAllControllersOff_Action = new JSONStorableAction("Set OFF all Controllers", () => {setAllControllersOff();applyCurrentControllerPreset();});
					RegisterAction(setAllControllersOff_Action);

					btn = CreateButton("Set ON only main Controllers");
					btn.button.onClick.AddListener(() =>
					{
						setMainControllerDefault();
					});
					JSONStorableAction setMainControllerDefault_Action = new JSONStorableAction("Set ON only main Controllers", () => {setMainControllerDefault();applyCurrentControllerPreset();});
					RegisterAction(setMainControllerDefault_Action);

					btn = CreateButton("Set ON only Rotation");
					btn.button.onClick.AddListener(() =>
					{
						setOnlyRotationOn();
					});
					JSONStorableAction setOnlyRotationOn_Action = new JSONStorableAction("Set ON only Rotation", () => {setOnlyRotationOn();applyCurrentControllerPreset();});
					RegisterAction(setOnlyRotationOn_Action);

					btn = CreateButton("Set ON Rotation for Main Controllers");
					btn.button.onClick.AddListener(() =>
					{
						setOnlyMainRotationOn();
					});
					JSONStorableAction setOnlyMainRotationOn_Action = new JSONStorableAction("Set ON Rotation for Main Controllers", () => {setOnlyMainRotationOn();applyCurrentControllerPreset();});
					RegisterAction(setOnlyMainRotationOn_Action);

					btn = CreateButton("Set ON only Hands");
					btn.button.onClick.AddListener(() =>
					{
						setOnlyHandsON();
					});
					JSONStorableAction setOnlyHandsON_Action = new JSONStorableAction("Set ON only Hands", () => {setOnlyHandsON();applyCurrentControllerPreset();});
					RegisterAction(setOnlyHandsON_Action);

					btn = CreateButton("Set ON only Feet");
					btn.button.onClick.AddListener(() =>
					{
						setOnlyFeetON();
					});
					JSONStorableAction setOnlyFeetON_Action = new JSONStorableAction("Set ON only Feet", () => {setOnlyFeetON();applyCurrentControllerPreset();});
					RegisterAction(setOnlyFeetON_Action);

					btn = CreateButton("Set ON only Hands and Feet");
					btn.button.onClick.AddListener(() =>
					{
						setOnlyHandsAndFeetON();
					});
					JSONStorableAction setOnlyHandsAndFeetON_Action = new JSONStorableAction("Set ON only Hands and Feet", () => {setOnlyHandsAndFeetON();applyCurrentControllerPreset();});
					RegisterAction(setOnlyHandsAndFeetON_Action);
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		//generate list of possible status for stringchooser
		protected List<string> getStatusList()
		{
			List<string> stringList = new List<string>();
			stringList.Add("On");
			stringList.Add("Off");
			stringList.Add("Following");
			stringList.Add("Hold");
			stringList.Add("Lock");
			stringList.Add("ParentLink");
			stringList.Add("PhysicsLink");
			stringList.Add("Comply");
			return stringList;
		}

		protected void getControllersList()
		{
			try 
			{
                //FreeControllerV3 FCv3 = new Dictionary<string, ForceReceiver>();
                foreach (FreeControllerV3 x in containingAtom.freeControllers)
                {
					if(!x.name.Contains("hair")&&!x.name.Contains("control"))
					{
						//SuperController.LogMessage("Controller found:"+x.name+"");
						FCv3_list.Add(x.name, true);
					}
                }
            }
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
		//function that creates UI elements at init
		protected void createUIControllerList()
		{
			List<string> stringList = new List<string>();
			stringList = getStatusList();

			JSONStorableBool tempJSONStorable;
			JSONStorableStringChooser tempJSONStringChooser;
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				//creating toggle
				var FCv3_enab = new JSONStorableBool(x.Key, x.Value,(bool enabled)=>
                {
                    FCv3_list[x.Key]=enabled;
                    //SuperController.LogMessage("updated RB:"+RB_elem.Key+" val:"+RB_enabled[RB_elem.Key]+"");
                });
				RegisterBool(FCv3_enab);
				CreateToggle(FCv3_enab,true);
				//creating scrollable popups to select status
				FreeControllerV3 FCv3 = containingAtom.GetStorableByID(x.Key) as FreeControllerV3;
				tempJSONStringChooser = new JSONStorableStringChooser(x.Key+"position", stringList, FCv3.currentPositionState.ToString(), "Position",(string newStatus)=>
				{
					FCv3_position_list[x.Key]=newStatus;
					//SuperController.LogMessage("updated RB:"+RB_elem.Key+" val:"+RB_enabled[RB_elem.Key]+"");
				});
				RegisterStringChooser(tempJSONStringChooser);
				CreateScrollablePopup(tempJSONStringChooser, true);
				tempJSONStringChooser = new JSONStorableStringChooser(x.Key+"rotation", stringList, FCv3.currentRotationState.ToString(), "Rotation",(string newStatus)=>
				{
					FCv3_rotation_list[x.Key]=newStatus;
					//SuperController.LogMessage("updated RB:"+RB_elem.Key+" val:"+RB_enabled[RB_elem.Key]+"");
				});
				RegisterStringChooser(tempJSONStringChooser);
				CreateScrollablePopup(tempJSONStringChooser, true);

				//Setting values in FCv3_position_list and FCv3_rotation_list
				FCv3_position_list.Add(x.Key,FCv3.currentPositionState.ToString());
				FCv3_rotation_list.Add(x.Key,FCv3.currentRotationState.ToString());
			}
		}

		public void updateUIControllerList(){
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				FreeControllerV3 FCv3 = containingAtom.GetStorableByID(x.Key) as FreeControllerV3;
				updateUIControllerPosition(x.Key,FCv3.currentPositionState.ToString());
				updateUIControllerRotation(x.Key,FCv3.currentRotationState.ToString());
			}
		}

		//funtions to update UI elements
		protected void updateUIControllerPosition(string controller, string newVal){
			FCv3_position_list[controller]=newVal;
			this.GetStringChooserJSONParam(controller+"position").val=newVal;
		}

		protected void updateUIControllerRotation(string controller, string newVal){
			//FCv3_position_list[controller]=newVal;
			this.GetStringChooserJSONParam(controller+"rotation").val=newVal;
		}

		protected void updateUIControllerEnabled(string controller, bool enabled)
		{
			//FCv3_list[controller]=enabled;
			if(FCv3_list.ContainsKey(controller))
			{
				this.GetBoolJSONParam(controller).val=enabled;
			}
		}

		//function to apply plugin preset to Person
		protected void applyCurrentControllerPreset()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				if(x.Value)
				{
					setControllerPositionState(x.Key,FCv3_position_list[x.Key]);
					setControllerRotationState(x.Key,FCv3_rotation_list[x.Key]);
				}
			}
		}

		protected void setControllerPositionState(string controller, string state)
        {
            FreeControllerV3 FCv3 = containingAtom.GetStorableByID(controller) as FreeControllerV3;
			//Possible states:
			//On, Off, Following, Hold, Lock, ParentLink, PhysicsLink, Comply

			switch (state)
			{
				case "On":
					FCv3.currentPositionState=FreeControllerV3.PositionState.On;
					break;
				case "Off":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Off;
					break;
				case "Following":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Following;
					break;
				case "Hold":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Hold;
					break;
				case "Lock":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Lock;
					break;
				case "ParentLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.ParentLink;
					break;
				case "PhysicsLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.PhysicsLink;
					break;
				case "Comply":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Comply;
					break;

				default:
					SuperController.LogError("setControllerPositionState: Unrecognized position status");
					break;
			}
        }

		protected void setControllerRotationState(string controller, string state)
        {
            FreeControllerV3 FCv3 = containingAtom.GetStorableByID(controller) as FreeControllerV3;
			//Possible states:
			//On, Off, Following, Hold, Lock, ParentLink, PhysicsLink, Comply

			switch (state)
			{
				case "On":
					FCv3.currentRotationState=FreeControllerV3.RotationState.On;
					break;
				case "Off":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Off;
					break;
				case "Following":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Following;
					break;
				case "Hold":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Hold;
					break;
				case "Lock":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Lock;
					break;
				case "ParentLink":
					FCv3.currentRotationState=FreeControllerV3.RotationState.ParentLink;
					break;
				case "PhysicsLink":
					FCv3.currentRotationState=FreeControllerV3.RotationState.PhysicsLink;
					break;
				case "Comply":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Comply;
					break;

				default:
					SuperController.LogError("setControllerRotationState: Unrecognized Rotation status");
					break;
			}
        }

		public void SavePreset(string aPath)
        {
            if (System.String.IsNullOrEmpty(aPath))
            {
                return;
            }
            _lastBrowseDir = aPath.Substring(0, aPath.LastIndexOfAny(new char[] { '/', '\\' })) + @"\";

            if (!aPath.ToLower().EndsWith("FreeControllerManager".ToLower()))
            {
                aPath += "." + "FreeControllerManager";
            }
            JSONClass saveJson = new JSONClass();
            saveJson["savedBy"] = "FreeControllerManager";

			List<string> keys = new List<string>();

			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				if(x.Value)
				{
					saveJson[x.Key]=FCv3_position_list[x.Key]+","+FCv3_rotation_list[x.Key];
					keys.Add(x.Key);
				}
			}
			saveJson["keyList"]=string.Join( ",", keys.ToArray());
            this.SaveJSON(saveJson, aPath);
        }

		void LoadPreset(string aPath)
        {
            if (System.String.IsNullOrEmpty(aPath))
            {
                return;
            }
            _lastBrowseDir = aPath.Substring(0, aPath.LastIndexOfAny(new char[] { '/', '\\' })) + @"\";
            var aJson = this.LoadJSON(aPath);

			string keys_string=aJson["keyList"];
			string[] keys=keys_string.Split(',');

			foreach(var x in keys)
			{
				string statuses = aJson[x];
				string[] words=statuses.Split(',');
				updateUIControllerPosition(x,words[0]);
				updateUIControllerRotation(x,words[1]);
			}
        }


		//preset functions

		protected void enableAllControllers()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				updateUIControllerEnabled(x.Key, true);
			}
		}

		protected void disableAllControllers()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				updateUIControllerEnabled(x.Key, false);	
			}
		}

		protected void setAllControllersOn()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				updateUIControllerPosition(x.Key, "On");
				updateUIControllerRotation(x.Key, "On");

			}
		}

		protected void setAllControllersOff()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				updateUIControllerPosition(x.Key, "Off");
				updateUIControllerRotation(x.Key, "Off");

			}
		}

		protected void setMainControllerDefault()
		{
			setAllControllersOff();
			updateUIControllerPosition("hipControl", "On");
			updateUIControllerPosition("headControl", "On");
			updateUIControllerPosition("chestControl", "On");
			updateUIControllerPosition("rHandControl", "On");
			updateUIControllerPosition("lHandControl", "On");
			updateUIControllerPosition("rFootControl", "On");
			updateUIControllerPosition("lFootControl", "On");
			updateUIControllerPosition("eyeTargetControl", "On");
			updateUIControllerRotation("hipControl", "On");
			updateUIControllerRotation("headControl", "On");
			updateUIControllerRotation("chestControl", "On");
			updateUIControllerRotation("rHandControl", "On");
			updateUIControllerRotation("lHandControl", "On");
			updateUIControllerRotation("rFootControl", "On");
			updateUIControllerRotation("lFootControl", "On");
			updateUIControllerRotation("eyeTargetControl", "On");
		}

		protected void setOnlyRotationOn()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				updateUIControllerPosition(x.Key, "Off");
				updateUIControllerRotation(x.Key, "On");

			}
			updateUIControllerPosition("eyeTargetControl", "On");
			updateUIControllerRotation("eyeTargetControl", "On");
			updateUIControllerPosition("rNippleControl", "Off");
			updateUIControllerRotation("rNippleControl", "Off");
			updateUIControllerPosition("lNippleControl", "Off");
			updateUIControllerRotation("lNippleControl", "Off");
			updateUIControllerPosition("testesControl", "Off");
			updateUIControllerRotation("testesControl", "Off");
			updateUIControllerPosition("penisBaseControl", "Off");
			updateUIControllerRotation("penisBaseControl", "Off");
			updateUIControllerPosition("penisMidControl", "Off");
			updateUIControllerRotation("penisMidControl", "Off");
			updateUIControllerPosition("penisTipControl", "Off");
			updateUIControllerRotation("penisTipControl", "Off");
			updateUIControllerPosition("lToeControl", "Off");
			updateUIControllerRotation("lToeControl", "Off");
			updateUIControllerPosition("rToeControl", "Off");
			updateUIControllerRotation("rToeControl", "Off");
		}

		protected void setOnlyMainRotationOn()
		{
			foreach(KeyValuePair<string,bool> x in FCv3_list)
			{
				setAllControllersOff();
				updateUIControllerRotation("hipControl", "On");
				updateUIControllerRotation("headControl", "On");
				updateUIControllerRotation("chestControl", "On");
				updateUIControllerRotation("rHandControl", "On");
				updateUIControllerRotation("lHandControl", "On");
				updateUIControllerRotation("rFootControl", "On");
				updateUIControllerRotation("lFootControl", "On");
				updateUIControllerRotation("eyeTargetControl", "On");
			}
		}
		

		protected void setOnlyHandsON()
		{
			setAllControllersOff();
			updateUIControllerPosition("rHandControl", "Lock");
			updateUIControllerPosition("lHandControl", "Lock");
			updateUIControllerRotation("rHandControl", "Lock");
			updateUIControllerRotation("lHandControl", "Lock");
		}

		protected void setOnlyFeetON(){
			setAllControllersOff();
			updateUIControllerPosition("rFootControl", "Lock");
			updateUIControllerPosition("lFootControl", "Lock");
			updateUIControllerRotation("rFootControl", "Lock");
			updateUIControllerRotation("lFootControl", "Lock");
		}

		protected void setOnlyHandsAndFeetON(){
			setAllControllersOff();
			updateUIControllerPosition("rHandControl", "Lock");
			updateUIControllerPosition("lHandControl", "Lock");
			updateUIControllerRotation("rHandControl", "Lock");
			updateUIControllerRotation("lHandControl", "Lock");
			updateUIControllerPosition("rFootControl", "Lock");
			updateUIControllerPosition("lFootControl", "Lock");
			updateUIControllerRotation("rFootControl", "Lock");
			updateUIControllerRotation("lFootControl", "Lock");
		}

		public UIDynamicTextField CreateTextInput(JSONStorableString jss, bool rightSide = false)
		{
			var textfield = CreateTextField(jss, rightSide);
			textfield.height = 20f;
			textfield.backgroundColor = Color.grey;
			var input = textfield.gameObject.AddComponent<InputField>();
			var rect = input.GetComponent<RectTransform>().sizeDelta = new Vector2(1f, 0.4f);
			input.textComponent = textfield.UItext;
			jss.inputField = input;
			return textfield;
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

	}
}